/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::gui-plugins::SkillManagerMonitorWidgetController
 * @author     Raphael Grimm ( raphael dot grimm at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <stack>
#include <ArmarXCore/core/system/ImportExportComponent.h>

#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>
#include <ArmarXGui/libraries/SimpleConfigDialog/SimpleConfigDialog.h>

#include <RobotAPI/interface/skills/SkillMemoryInterface.h>

#include <RobotAPI/gui-plugins/SkillManagerPlugin/ui_SkillManagerMonitorWidget.h>

#include "../AronTreeWidgetItem.h"

#include <RobotAPI/libraries/aron/core/type/variant/All.h>
#include <RobotAPI/libraries/aron/core/data/variant/All.h>
#include <RobotAPI/libraries/aron/core/type/visitor/variant/VariantVisitor.h>

namespace armarx
{
    // Convert aron type to tree widget
    class AronTreeWidgetCreatorVisitor :
            public armarx::aron::type::ConstVariantVisitor
    {
    public:
        AronTreeWidgetItem* createdQWidgetItem;

        AronTreeWidgetCreatorVisitor() = default;

        void createSimpleTreeViewWidget(Input& i, const std::string&);

        void visitAronVariant(const aron::type::ObjectPtr&) override;
        void visitAronVariant(const aron::type::DictPtr& i) override;
        void visitAronVariant(const aron::type::PairPtr& i) override;
        void visitAronVariant(const aron::type::TuplePtr& i) override;
        void visitAronVariant(const aron::type::ListPtr& i) override;
        void visitAronVariant(const aron::type::NDArrayPtr& i) override;
        void visitAronVariant(const aron::type::MatrixPtr& i) override;
        void visitAronVariant(const aron::type::QuaternionPtr& i) override;
        void visitAronVariant(const aron::type::PositionPtr& i) override;
        void visitAronVariant(const aron::type::OrientationPtr& i) override;
        void visitAronVariant(const aron::type::PosePtr& i) override;
        void visitAronVariant(const aron::type::ImagePtr& i) override;
        void visitAronVariant(const aron::type::PointCloudPtr& i) override;
        void visitAronVariant(const aron::type::IntEnumPtr& i) override;
        void visitAronVariant(const aron::type::IntPtr& i) override;
        void visitAronVariant(const aron::type::LongPtr& i) override;
        void visitAronVariant(const aron::type::FloatPtr& i) override;
        void visitAronVariant(const aron::type::DoublePtr& i) override;
        void visitAronVariant(const aron::type::BoolPtr& i) override;
        void visitAronVariant(const aron::type::StringPtr& i) override;
        void visitAronVariant(const aron::type::TimePtr& i) override;
        void visitUnknown(Input&) override;
    };
}


