/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::SkillManager
 * @author     Raphael Grimm ( raphael dot grimm at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/interface/aron.ice>


module armarx
{
    module skills
    {
        module callback
        {
            module dti
            {
                interface SkillProviderCallbackInterface;
            }
        }
    }
}


module armarx
{
    module skills
    {
        module provider
        {
            module dto
            {
                sequence<string> StringList;

                // Description of a skill. Needs to be sent to a skill manager or memory
                // A skill is nothing but a executable thing, which can be executed on one or more 'targets' (empty means all)
                struct SkillDescription
                {
                    string                        skillName;     // the name of the skill
                    string                        description;   // a human readable description of what the skill does
                    StringList                    targets;       // see above
                    long                          timeoutMs;     // in milliseconds
                    aron::type::dto::AronObject   acceptedType;  // the name of the object is irrelevant and only used in GUI
                };
                dictionary<string, SkillDescription> SkillDescriptionMap;

                // Inputs
                struct SkillExecutionInfo
                {
                    string skillName;
                    aron::data::dto::Dict params;
                    callback::dti::SkillProviderCallbackInterface* callbackInterface; // use nullptr if you do not want to have callbacks
                    bool waitUntilSkillFinished;
                };

                // The status enum of a skill
                module Execution
                {
                    enum Status
                    {
                        Idle,
                        Scheduled,
                        Running,

                        // terminating values
                        Failed,
                        Succeeded,
                        Aborted
                    };
                }

                // Status updates of a skill
                struct SkillStatusUpdate
                {
                    string                                         providerName;
                    string                                         skillName;
                    aron::data::dto::Dict                          usedParams;
                    callback::dti::SkillProviderCallbackInterface* usedCallbackInterface;
                    Execution::Status                              status;
                    aron::data::dto::Dict                          data; // can be anything
                };

                dictionary<string, SkillStatusUpdate> SkillStatusUpdateMap;
            }

            module dti
            {

                interface SkillProviderInterface
                {
                    dto::SkillDescriptionMap getSkills();
                    dto::SkillStatusUpdate getSkillExecutionStatus(string name);
                    void executeSkill(dto::SkillExecutionInfo executionInfo);
                    void abortSkill(string skill, bool waitUntilSkillFinished);
                };

                dictionary<string, SkillProviderInterface*> SkillProviderMap;
            }
        }
    }
}

module armarx
{
    module skills
    {
        module callback
        {
            module dti
            {
                interface SkillProviderCallbackInterface
                {
                    // used for callbacks from providers to update their skill execution status
                    void updateStatusForSkill(provider::dto::SkillStatusUpdate statusUpdate);
                }
            }
        }
    }
}

