#pragma once

#include <shared_mutex>

#include "../SkillDescription.h"
#include "../SkillStatusUpdate.h"
#include "../Skill.h"

#include <RobotAPI/interface/skills/SkillManagerInterface.h>

namespace armarx
{
    namespace skills
    {
        namespace detail
        {
            class SkillImplementationWrapper
            {
            public:
                // fixed values. Do not change after skill instantiation
                const std::unique_ptr<Skill> skill;

                // Current execution status. Changes during execution
                // The status also holds the used parameterization
                // skillName and providerName are const after registering the skill in a provider
                mutable std::shared_mutex skillStatusMutex;
                SkillStatusUpdate statusUpdate;

                // Task information. task is recreated every time the skill restarts
                mutable std::shared_mutex executingMutex;
                std::thread task;

                SkillImplementationWrapper(std::unique_ptr<skills::Skill>&& skill) :
                    skill(std::move(skill))
                {
                    ARMARX_CHECK_NOT_NULL(this->skill);
                }

                // execute a skill. The parameterization is copied
                void execute(const skills::SkillParameterization);
            };
        }
    }
}
